// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file RosenbrockAdaptInstationary.h */

#ifndef AMDIS_ROSENBROCKADAPTINSTATIONARY_H
#define AMDIS_ROSENBROCKADAPTINSTATIONARY_H

#include "AMDiS_fwd.h"
#include "time/RosenbrockStationary.h"
#include "time/RosenbrockMethod.h"
#include "AdaptInstationary.h"
#include "CreatorMap.h"

namespace AMDiS {

  class RosenbrockAdaptInstationary : public AdaptInstationary
  {
  public:
    RosenbrockAdaptInstationary(std::string name, 
				RosenbrockStationary *problemStat,
				AdaptInfo *info,
				ProblemTimeInterface *problemInstat,
				AdaptInfo *initialInfo,
				time_t initialTimestamp = 0);

    void oneTimestep();

    double* getTauGamma()
    {
      return &tauGamma;
    }

  protected:
    RosenbrockMethod *rosenbrockMethod;

    RosenbrockStationary *rosenbrockStat;

    bool firstTimestep, lastTimestepRejected, succRejection;

    double tauAcc, estAcc, tauRej, estRej;

    double tauGamma;
  };
 
}

#endif // AMDIS_ROSENBROCKADAPTINSTATIONARY_H
